<?php
/**
 * Copyright © 2016 MageWorx. All rights reserved.
 * See LICENSE.txt for license details.
 */
namespace MageWorx\SeoUrls\Helper;

use Magento\Catalog\Model\Layer\FilterableAttributeListInterface;
use Magento\Framework\App\Helper\AbstractHelper;

/**
 * SEO URLs layer helper
 */
class Layer extends AbstractHelper
{
    /**
     * Core data
     *
     * @var \Magento\Framework\Filter\FilterManager
     */
    protected $filter;

    /**
     * @var FilterableAttributeListInterface
     */
    protected $filterableAttributes;

    /**
     * @var array|null
     */
    protected $loadedAttributesData = null;

    /**
     * Layer constructor.
     * @param FilterableAttributeListInterface $filterableAttributes
     */
    public function __construct(
        \Magento\Framework\Filter\FilterManager $filter,
        \Magento\Catalog\Model\Layer\FilterableAttributeListInterface $filterableAttributes
    ) {
        $this->filter = $filter;
        $this->filterableAttributes = $filterableAttributes;
    }


    /**
     * @param array $layerParams
     * @return array
     */
    public function parseLayeredParams($layerParams)
    {
        $paramsFilteredByAttribute = [];

        foreach ($layerParams as $code => $value) {
            if ($this->isHiddenAttribute($code)) {
                if ($data = $this->getHiddenOptionDataByAttribute($value)) {
                    $paramsFilteredByAttribute[$data['attribute_code']] = $data['option_id'];
                }
            } else {
                if ($data = $this->getNotHiddenOptionDataByAttribute($code, $value)) {
                    $paramsFilteredByAttribute[$data['attribute_code']] = $data['option_id'];
                }
            }
        }
        return $paramsFilteredByAttribute;
    }

    /**
     * @param $value
     * @return string|false
     */
    protected function getHiddenOptionDataByAttribute($value)
    {
        $attr = $this->getFilterableAttributes();

        foreach ($attr as $attrCode => $attrData) {
            foreach ($attrData['options'] as $optionId => $optionLabel) {
                if ($value == $this->formatUrlKey($optionLabel)) {
                    return ['option_id' => $optionId, 'attribute_code' => $attrCode];
                }
            }
        }

        return false;
    }

    /**
     * @param string $code
     * @param string $value
     * @return array
     */
    protected function getNotHiddenOptionDataByAttribute($code, $value)
    {
        if ($code == 'price') {
            if (strpos($value, '-') !== false) {
                $multipliers = explode('-', $value);
                $priceFrom   = floatval($multipliers[0]);
                $priceTo     = $multipliers[1] ? floatval($multipliers[1]) : '';
                $value       = $priceFrom . '-' . $priceTo;

                return ['option_id' => $value, 'attribute_code' => $code];
            }
        } else {
            $attr = $this->getFilterableAttributes();

            if (isset($attr) && !empty($attr)) {
                $code = str_replace('-', '_', $code); // attrCode is only = [a-z0-9_]

                if (isset($attr[$code])) {
                    foreach ($attr[$code]['options'] as $optionId => $optionLabel) {
                        if ($value == $this->formatUrlKey($optionLabel)) {
                            return ['option_id' => $optionId, 'attribute_code' => $code];
                        }
                    }
                }
            }
        }
        return false;
    }

    /**
     * @param $param
     * @return bool
     */
    protected function isHiddenAttribute($param)
    {
        return (is_numeric($param));
    }

    /**
     * @return bool
     */
    protected function isHiddenPriceAttribute()
    {
        return false;
    }

    /**
     * @return array|null
     */
    public function getFilterableAttributes()
    {
        if ($this->loadedAttributesData !== null) {
            return $this->loadedAttributesData;
        }

        $attributesData = [];
        $attributeCollection = $this->filterableAttributes->getList();

        foreach ($attributeCollection as $attribute) {
            $attributeCode = $attribute->getAttributeCode();

            $attributesData[$attributeCode]['type'] = $attribute->getBackendType();
            $options = $attribute->getSource()->getAllOptions();

            foreach ($options as $option) {
                //$attributesData[$attributeCode]['options'][$this->formatUrlKey($option['label'])] = $option['label'];
                $attributesData[$attributeCode]['options'][$option['value']] = $option['label'];
                //$attributesData[$attributeCode]['options'][$option['value']] = $this->getOptionLabel($option['label'], $option['value']);
                $attributesData[$attributeCode]['frontend_label'] = $attribute->getFrontendLabel();
            }
        }


        $this->loadedAttributesData = $attributesData;
        return $this->loadedAttributesData;
    }
    /*
    * @param string $optionLabel
    * @param string $optionValue
    * return string
    */
    protected function getOptionLabel($optionLabel, $optionValue)
    {
        $translatedLabel = $this->filter->translitUrl($optionLabel);
        if (!$translatedLabel) {
            $translatedLabel = $optionValue;
        } elseif (in_array($translatedLabel, $this->optionLabelList)) {
            return $this->getOptionLabel($translatedLabel . '-1', $optionValue);
        }
        $this->optionLabelList[] = $translatedLabel;

        return $translatedLabel;
    }

    /**
     * Format URL key from name or defined key
     *
     * @param string $str
     * @return string
     */
    public function formatUrlKey($str)
    {
        return $this->filter->translitUrl($str);
    }
}
